import json
import re
import sys


defaul_tags =  {
    'tagStartCode': "START CODE HERE",
    'tagEndCode': "END CODE HERE",
	'tagStartOmitBlock': "START OMIT BLOCK",
	'tagEndOmitBlock': "END OMIT BLOCK",
	'tagOmit': "@OMIT",
	'tagKeep': "@KEEP",
	'tagReplaceEquals': "@REPLACE EQUALS",
	'tagReplace': "@REPLACE",
}

def learner_generator(input, tags = defaul_tags, keep_unit_tests=True):
    ''' Creates the Assignment and the Solution notebooks from a development notebook.
    The function will create new files in the same folder where the input is located.
    Args:
        'input' (str): The path to the _dev.ipynb notebook.
        'keep_unit_tests' (bool): Keep unit tests in the assignments notebook?
    Examples:
        'deploy_assignment("C1_W1_Assignment_dev.ipynb", True)'
        # Produces: 'C1_W1_Assignment.ipynb'
                    'C1_W1_Assignment_Solution.ipynb'
    '''

    try:

        with open(input) as f:
            nb_assignment = json.load(f)

        # For the assignment notebook
        cells2 = nb_assignment['cells']
        for cell in cells2:
            code_block = False
            omit_block = False
            if cell['cell_type'] == 'code':
                source = cell['source']
                line_number = -1
                for ln in source:
                    line_number += 1
                    if tags['tagStartCode'] in ln:
                        code_block = True
                        next

                    if tags['tagEndCode'] in ln:
                        code_block = False
                        next

                    if tags['tagStartOmitBlock'] in ln:
                        omit_block = True
                        next

                    if tags['tagEndOmitBlock'] in ln:
                        omit_blocklock = False
                        next

                    if omit_block or tags['tagOmit'] in ln:
                        next
                    
                    if code_block and not omit_block:
                        # Example: 'k = b[10] # @KEEP'   => 'k = b[10]'
                        if tags['tagKeep'] in ln:
                            source[line_number] = re.split("#[\ ]*"+tags['tagKeep'], ln)[0]
                            continue
                        # Example: 'k = b[10] # @REPLACE EQUALS b[None]'   => 'k = b[None]'
                        if tags['tagReplaceEquals'] in ln:
                            splitted = re.split("#[\ ]*" + tags['tagReplaceEquals'] + '[\ ]*', ln)
                            new_val = splitted[1]
                            splitted_eq = splitted[0].split("=")
                            source[line_number] = splitted_eq[0] + "= " + new_val
                            continue
                        # Example: 'k = b[10] # @REPLACE x = b[None]'   => 'x = b[None]'
                        if tags['tagReplace'] in ln:
                                ind = get_indentation(ln)
                                splitted = re.split("#[\ ]*" + tags['tagReplace'] + '[\ ]*', ln)
                                source[line_number] =  ind + splitted[1]
                                continue
                        # Example: 'k = b[10] # Comment'   => 'k = None'
                        if "=" in ln:
                                splitted = re.split("=", ln)
                                source[line_number] = splitted[0] + "= None\n"
                                continue
        print('dsdsdd')
        if '_Dev'in input:
            with open(input.replace('_Dev', ''), 'w') as file:
                file.write(json.dumps(nb_assignment, indent=2))
        else:
            with open(input.replace('.ipynb', '_Assignment.ipynb'), 'w') as file:
                file.write(json.dumps(nb_assignment, indent=2))

    except:
        print("Error during deployment")
        #print(ValueError)

def get_indentation(ln):
	return re.search('^[\ ]+', ln).group(0)

if __name__ == "__main__":
    if sys.argv[1] == 'help':
        print(f'Usage: \n dlai_tools.learner_generator input_file.ipynb [tags.json]')
    else:
        tags = defaul_tags
        if len(sys.argv) > 2 and '.json' in sys.argv[2]:
            with open(sys.argv[2]) as f:
                tags = json.load(f)
        learner_generator(sys.argv[1], tags)


from testing_utils import *
import unittest
import numpy as np

# Mainteiner: Andres Castillo
# Owner: DeepLearning.AI

class TestingUtilsTestCase(unittest.TestCase):

    def test_datatype_check(self):
        assert datatype_check("abc", "csf", "Error") == 1
        assert datatype_check([1, 2, 3], [4, 3, 5], "Error") == 1
        assert datatype_check([1, 2, 3], [4., 3., 5.], "Error", level=1) == 0
        assert datatype_check([], [1, 2, 3], "Error") == 1
        assert datatype_check([1, 2, 3], [], "Error") == 1
        assert datatype_check(np.array([1, 2, 3]), [], "Error", level=1) == 0
        assert datatype_check([1, 2, 3], [], "Error", level=1) == 0
        assert datatype_check(np.array([1, 2, 3]), [1, 2, 3], "Error") == 0
        assert datatype_check([1, 2, 3], np.array([1, 2, 3]), "Error") == 0

            
    def test_equation_output_check(self):
        assert equation_output_check("abc", "abc", "Error") == 1
        assert equation_output_check([1, 2, 3], [1, 2, 3], "Error") == 1
        assert equation_output_check([[1, 2, 3], [4, 5, 6], [9., 10., 11.]], np.array([[1., 2, 3.], [4, 5, 6], [9, 10, 11]]), "Error") == 1
        assert equation_output_check((1.0, 2.0, 3), [1, 2, 3], "Error") == 1
        assert equation_output_check([1, 2, 3], [], "Error") == 0
        assert equation_output_check([1, 2, 3], [2., 3., 4.], "Error") == 0
        assert equation_output_check(np.array([1, 2, 3]), [1, 2., 3], "Error") == 1
        assert equation_output_check(np.array([]), [1, 2, 3], "AssertionError 1") == 0
        assert equation_output_check([], [1, 2, 3], "AssertionError 1") == 0
        assert equation_output_check("abc", [1, 2, 3], "AssertionError 1") == 0
        assert equation_output_check([1, 2, 3], "abc",  "AssertionError 1") == 0
        assert equation_output_check("abc", "xyz", "AssertionError 2") == 0

    def test_shape_check(self):
        assert shape_check("abc", "csf", "Error") == 1 # It does not have sence, but I include it as reference
        assert shape_check("abc", "csfsss", "Error") == 1 # It does not have sence, but I include it as reference
        assert shape_check([1, 2, 3], [4, 3, 5], "Error") == 1
        assert shape_check(np.array([1, 2, 3]), np.array([4, 3, 5]), "Error") == 1
        assert shape_check([], [1, 2, 3], "Error") == 0
        assert shape_check([1, 2, 3], [], "Error") == 0
        assert shape_check([3, 6, 7], np.array([1, 2, 3]), "Error") == 1
        assert shape_check([[3, 2], [6, 0], [7, 7]], np.array([[3, 2], [6, 0], [7, 7]]), "Error") == 1
        assert shape_check(np.array([3, 6, 7]), [1, 2, 3], "Error") == 1
        assert shape_check(np.array([[3, 2], [6, 0], [7, 7]]), np.array([[3], [6], [7]]), "Error") == 0


if __name__ == '__main__':
    unittest.main()




import json
import re
import sys

default_patterns = {
    'section': ['^## [0-9]+ - '], # '- [Part {section_number}: {section_title}](# {section_number})'
    'subsection': ['^### [0-9]+.[0-9]+ '], 
    'exercise': ['^### Exercise [0-9]+'], 
    'toc_title': ['^# Outline']
}

default_templates = {
    'section': '## {section_number} - {section_title}', 
    'subsection': '### {section_number}.{subsection_number} {subsection_title}', 
    'exercise': '### Exercise {exercise_number}', 
    'toc_title': '# Outline'
}

def add_toc(input, patterns=default_patterns, templates=default_templates):
    """ Function to include a table of content on a jupyter notebook based on its
        markdown. It receives a file path as input, and creates a new notebook
        file that includes the table of content in the same path.
        It performs some style fix during the process. For more information see
        the attached notebook example.

    Args:
        input ([strin]): The notebook file to be formated
    """
    try:
        with open(input) as f:
            notebook = json.load(f)

        cells = notebook['cells']

        head_cell = cells[0]

        variables = {'section_number': 0,
             'section_title': '',
             'subsection_number': 0,
             'subsection_title': '',
             'exercise_number': 0}

        toc = ['\n', templates['toc_title'] + '\n']

        for cell in cells[1: len(cells)]:
            if cell['cell_type'] == 'markdown':
                source_array = cell['source']
                row = 0
                skip = False
                for source in source_array:
                    print(f"{row} - {source}")
                    if not skip:
                        if re.search('^[#]+ Introduction', source, re.IGNORECASE):

                            html_tag = '<a name="0"></a>\n'
                            if row > 0 and source_array[row - 1].find('<a name=') >= 0:
                                source_array[row - 1] = html_tag
                            else:
                                source_array.insert(row, html_tag)
                                row += 1
                                skip = True

                            toc.append('- [Introduction](#0)\n')
                            source_array[row] = '# Introduction\n'
                            

                        if match_patterns(source, 'section', patterns):
                            variables['section_number'] += 1
                            variables['section_title'] = get_title(source, 'section', patterns)
                            html_tag = f'<a name="{variables["section_number"]}"></a>\n'

                            if row > 0 and source_array[row - 1].find('<a name=') >= 0:
                                source_array[row - 1] = html_tag
                            else:
                                source_array.insert(row, html_tag)
                                row += 1
                                skip = True

                            md_line = templates['section'].format(**variables)
                            toc.append(f'- [{md_line.replace("#", "")}](#{variables["section_number"]})\n')

                            # Fix source line if needed
                            source_array[row] = md_line + '\n'

                            variables['subsection_number'] = 0
                            #skip = True

                        if match_patterns(source, 'subsection', patterns):
                            variables['subsection_number'] += 1  # Increase section

                            label = f"{variables['section_number']}.{variables['subsection_number']}"
                            variables['subsection_title'] = get_title(source, 'subsection', patterns)
                            html_tag = f'<a name="{label}"></a>\n'
                            print(f"--- {row} - {source}")
                            print(f"{variables['section_number']}- {variables['subsection_number']}")
                            print(row)
                            if row > 0 and source_array[row - 1].find('<a name=') >= 0:
                                source_array[row - 1] = html_tag
                            else:
                                source_array.insert(row, html_tag)
                                row += 1
                                skip = True


                            md_line = templates['subsection'].format(**variables)
                            toc.append(f'  - [{md_line.replace("#", "")}](#{label})\n')
                            # Fix source line if needed
                            source_array[row] = f'{md_line}\n'

                        if match_patterns(source,'exercise', patterns):
                            variables['exercise_number'] += 1
                            label = "{:02d}".format(variables['exercise_number'])
                            html_tag = '<a name="ex' + label + '"></a>\n'

                            if row > 0 and source_array[row - 1].find('<a name=') >= 0:
                                source_array[row - 1] = html_tag
                            else:
                                source_array.insert(row, html_tag)
                                row += 1
                                skip = True

                            ident = '    '
                            if variables['subsection_number'] == 0:  # There is not subsection
                                ident = '  '

                            md_line = templates['exercise'].format(**variables)
                            toc.append(f'{ident}- [{md_line.replace("#", "")}](#ex{label})\n')

                            source_array[row] = f'{md_line}\n'
                            #skip = True
                    else:
                        skip = False
                        row -= 1
                    row += 1
            # Clear outputs and reset the execution count
            if cell['cell_type'] == 'code':
                cell['execution_count'] = 0
                cell['outputs'] = []

        # Remove the old toc
        toc_line = 0
        for line in head_cell['source']:
            if re.search('^' + patterns['toc_title'][0], line, re.IGNORECASE):
                head_cell['source'] = head_cell['source'][0: toc_line]
                break
            toc_line += 1

        head_cell['source'] = head_cell['source'] + toc

        with open(input.replace('.ipynb', '_toc.ipynb'), 'w') as file:
            file.write(json.dumps(notebook, indent=2))

    except ValueError:
        print("Error during deployment")
        print(ValueError)


def match_patterns(line, name, patterns):
    for pattern in patterns[name]:
        if re.search(pattern, line, re.IGNORECASE):
            return True
    return False


def get_title(line, name, patterns):
    for pattern in patterns[name]:
        title_search = re.search(pattern, line, re.IGNORECASE)
        if title_search:
            return re.sub(pattern, '', line).replace('\n', '')
    return ''

if __name__ == "__main__":
    if sys.argv[1] == 'help':
        print(f'Usage: \n dlai_tools.add_toc input_file.ipynb [patterns.json templates.json]')
    else:
        patterns = default_patterns
        templates = default_templates
        if len(sys.argv) > 3 and '.json' in sys.argv[2]:
            with open(sys.argv[2]) as f:
                patterns = json.load(f)
            with open(sys.argv[3]) as f:
                templates = json.load(f)

        add_toc(sys.argv[1], patterns, templates)


from tensorflow.keras.layers import Input
from tensorflow.keras.layers import Conv2D
from tensorflow.keras.layers import MaxPooling2D
from tensorflow.keras.layers import Dropout 
from tensorflow.keras.layers import Conv2DTranspose
from tensorflow.keras.layers import concatenate
from tensorflow.keras.layers import ZeroPadding2D
from tensorflow.keras.layers import Dense
from tensorflow.keras.layers import LSTM
from tensorflow.keras.layers import RepeatVector
from tensorflow.keras.layers import TimeDistributed
from tensorflow.keras.layers import GRU
from tensorflow.keras.layers import Conv1D

import numpy as np
from termcolor import colored
from copy import deepcopy

# Compare the two inputs
def comparator(learner, instructor):
    layer = 0
    if len(learner) != len(instructor):
        raise AssertionError(f"The number of layers in the model is incorrect. Expected: {len(instructor)} Found: {len(learner)}") 
    for a, b in zip(learner, instructor):
        if tuple(a) != tuple(b):
            print(colored("Test failed", attrs=['bold']),
                  f"at layer: {layer}",
                  "\n Expected value \n\n", colored(f"{b}", "green"), 
                  "\n\n does not match the input value: \n\n", 
                  colored(f"{a}", "red"))
            raise AssertionError("Error in test") 
        layer += 1
    print(colored("All tests passed!", "green"))
    return True

# extracts the description of a given model
def summary(model):
    model.compile(optimizer='adam',
                  loss='categorical_crossentropy',
                  metrics=['accuracy'])
    result = []
    for layer in model.layers:
        descriptors = [layer.__class__.__name__, layer.output_shape, layer.count_params()]
        if (type(layer) == Conv1D):
            descriptors.append(layer.padding)
            descriptors.append(layer.activation.__name__)
            descriptors.append(layer.strides)
            descriptors.append(layer.kernel_size)
            descriptors.append(layer.kernel_initializer.__class__.__name__)
           
        if (type(layer) == Conv2D):
            descriptors.append(layer.padding)
            descriptors.append(layer.activation.__name__)
            descriptors.append(layer.kernel_initializer.__class__.__name__)
            
        if (type(layer) == MaxPooling2D):
            descriptors.append(layer.pool_size)
            descriptors.append(layer.strides)
            descriptors.append(layer.padding)
            
        if (type(layer) == Dropout):
            descriptors.append(layer.rate)
            
        if (type(layer) == ZeroPadding2D):
            descriptors.append(layer.padding)
            
        if (type(layer) == Dense):
            descriptors.append(layer.activation.__name__)
            
        if (type(layer) == LSTM):
            descriptors.append(layer.input_shape)
            descriptors.append(layer.activation.__name__)
            
        if (type(layer) == RepeatVector):
            descriptors.append(layer.n)
            
        if (type(layer) == TimeDistributed):
            descriptors.append(layer.layer.activation.__name__)  
            
        if (type(layer) == GRU):
            descriptors.append(layer.return_sequences)    
            
        result.append(descriptors)
    return result


def datatype_check(expected_output, target_output, error, level=0):
    success = 0
    if (level == 0):
        try:
            assert isinstance(target_output, type(expected_output))
            return 1
        except:
            return 0
    else:
        if isinstance(expected_output, tuple) or isinstance(expected_output, list) \
                or isinstance(expected_output, np.ndarray) or isinstance(expected_output, dict):
            if isinstance(expected_output, dict):
                range_values = expected_output.keys()
            else:
                range_values = range(len(expected_output))
            if len(expected_output) != len(target_output) or not isinstance(target_output, type(expected_output)):
                return 0
            for i in range_values:
                try:
                    success += datatype_check(expected_output[i],
                                            target_output[i], error, level - 1)
                except:
                    print("Error: {} in variable {}, expected type: {}  but expected type {}".format(error,
                                                                                                    i,
                                                                                                    type(
                                                                                                        target_output[i]),
                                                                                                    type(expected_output[i]
                                                                                                        )))
            if success == len(expected_output):
                return 1
            else:
                return 0

        else:
            try:
                assert isinstance(target_output, type(expected_output))
                return 1
            except:
                return 0


def equation_output_check(expected_output, target_output, error):
    success = 0
    if isinstance(expected_output, tuple) or isinstance(expected_output, list) or isinstance(expected_output, dict):
        if isinstance(expected_output, dict):
            range_values = expected_output.keys()
        else:
            range_values = range(len(expected_output))

        if len(expected_output) != len(target_output):
                return 0

        for i in range_values:
            try:
                success += equation_output_check(expected_output[i],
                                                 target_output[i], error)
            except:
                print("Error: {} for variable in position {}.".format(error, i))
        if success == len(expected_output):
            return 1
        else:
            return 0

    else:
        try:
            if hasattr(expected_output, 'shape'):
                np.testing.assert_array_almost_equal(
                    target_output, expected_output)
            else:
                assert target_output == expected_output
        except:
            return 0
        return 1


def shape_check(expected_output, target_output, error):
    success = 0
    if isinstance(expected_output, tuple) or isinstance(expected_output, list) or \
            isinstance(expected_output, dict) or isinstance(expected_output, np.ndarray):
        if isinstance(expected_output, dict):
            range_values = expected_output.keys()
        else:
            range_values = range(len(expected_output))

        if len(expected_output) != len(target_output):
                return 0
        for i in range_values:
            try:
                success += shape_check(expected_output[i],
                                       target_output[i], error)
            except:
                print("Error: {} for variable {}.".format(error, i))
        if success == len(expected_output):
            return 1
        else:
            return 0

    else:
        return 1


def single_test(test_cases, target):
    success = 0
    for test_case in test_cases:
        try:
            if test_case['name'] == "datatype_check":
                assert isinstance(target(*test_case['input']),
                                  type(test_case["expected"]))
                success += 1
            if test_case['name'] == "equation_output_check":
                assert np.allclose(test_case["expected"],
                                   target(*test_case['input']))
                success += 1
            if test_case['name'] == "shape_check":
                assert test_case['expected'].shape == target(
                    *test_case['input']).shape
                success += 1
        except:
            print("Error: " + test_case['error'])

    if success == len(test_cases):
        print("\033[92m All tests passed.")
    else:
        print('\033[92m', success, " Tests passed")
        print('\033[91m', len(test_cases) - success, " Tests failed")
        raise AssertionError(
            "Not all tests were passed for {}. Check your equations and avoid using global variables inside the function.".format(target.__name__))


def multiple_test(test_cases, target):
    success = 0
    for test_case in test_cases:
        try:
            test_input = deepcopy(test_case['input'])
            target_answer = target(*test_input)
        except:
            print('\33[30m', "Error, interpreter failed when running test case with these inputs: " + 
                  str(test_input))
            raise AssertionError("Unable to successfully run test case.".format(target.__name__))

        try:
            if test_case['name'] == "datatype_check":
                success += datatype_check(test_case['expected'],
                                      target_answer, test_case['error'])
            if test_case['name'] == "equation_output_check":
                success += equation_output_check(
                    test_case['expected'], target_answer, test_case['error'])
            if test_case['name'] == "shape_check":
                success += shape_check(test_case['expected'],
                                   target_answer, test_case['error'])
        except:
            print('\33[30m', "Error: " + test_case['error'])

    if success == len(test_cases):
        print("\033[92m All tests passed.")
    else:
        print('\033[92m', success, " Tests passed")
        print('\033[91m', len(test_cases) - success, " Tests failed")
        raise AssertionError(
            "Not all tests were passed for {}. Check your equations and avoid using global variables inside the function.".format(target.__name__))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()



setuptools.setup(
    name="dlai_tools",  # Replace with your own username
    version="0.5.4",
    author="Andrés Castillo",
    author_email="andcastillo@gmail.com",
    description="Deployment script for notebooks",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/https-deeplearning-ai/dlai_tools",
    download_url = 'https://github.com/https-deeplearning-ai/dlai_tools/releases/tag/0.5.4',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
